import time
from kafka import KafkaConsumer


class KafkaListener:
    def __init__(self, **kwargs):
        """
        :param topic, kafka_ip, kafka_port, group_id
        """
        self.topic, self.kafka_ip, self.kafka_port, self.group_id = kwargs['topic'], kwargs['kafka_ip'], kwargs['kafka_port'], kwargs['group_id']
        self.consumer = KafkaConsumer(self.topic, bootstrap_servers= f'{self.kafka_ip}:{self.kafka_port}', group_id = self.group_id)
        print('succeed in connecting to kafka')


    def connect(self):
        for _ in range(120):
            try:
                consumer = KafkaConsumer(self.topic, bootstrap_servers= f'{self.kafka_ip}:{self.kafka_port}', group_id = self.group_id)
                return consumer
            except BaseException:
                print('connection kafka failed, retrying ...')
                time.sleep(1)
    
    def __iter__(self):
        return self.consumer


import pymysql
import time


class DataBase:
    def __init__(self, db_ip='localhost', db='recommend_system', user_name='root', password='root', port=3306):
        self.db_ip, self.db, self.user_name, self.password, self.port = db_ip, db, user_name, password, port
        self.cursor = self.connect()
        self.db_name = db


    def connect(self):
        """
        :param dp_ip: the ip address of database
        :param db: the database name
        :param user_name: the user name to login the database
        :param port: the port to connect the database
        :return the cursor of connection. if connection failed, return None
        """
        for i in range(10):#try 10 times if connection failed
            try:
                self.connection = pymysql.connect(host = self.db_ip, user = self.user_name,password = self.password, port = self.port, db = self.db)
                return self.connection.cursor()
            except pymysql.err.OperationalError:
                print('database connection failed: retrying ...')
                time.sleep(1)
    
    def create_table(self,table_cmd):
        """
        :param table_cmd: the commends to create table
        """
        if self.cursor is None:
            self.cursor = self.connect() #reconnect
        if self.cursor is None:
            print('failed to create table, connection failed')
            return
        if table_cmd == "":
            return
        self.cursor.execute(f'use {self.db_name}')
        self.cursor.execute(table_cmd)
    
    def save_table(self,data,table_name):
        """
        :param data: column_name -> [value1, value2 ...]
        """
        if self.cursor is None:
            self.cursor = self.connect()
        if self.cursor is None:
            print('failed to save table, connection failed')
            return
        if data is None or len(data)==0 or table_name is None or table_name == "": 
            return
        for i in range(len(list(data.values())[0])):
            info = {k:v[i] for k,v in data.items()}
            keys = ', '.join(info.keys())
            values = ', '.join(['%s'] * len(info))
            cmd = 'INSERT INTO {table}({keys}) VALUES ({values}) ON DUPLICATE KEY UPDATE'.format(table=table_name, keys=keys, values=values)
            params = list(info.values())+list(info.values())
            params = [str(i) for i in params]
            params = ['null' if i == '' else i for i in params]
            params = ['\'' + i + '\'' if i != 'null' else i for i in params]
            update = ','.join([" {key} = %s".format(key=key) for key in info])
            cmd += update
            cmd = cmd%tuple(params)
            for _ in range(5):
                try:
                    self.cursor.execute(cmd)
                    self.connection.commit()
                    break
                except BaseException as e:
                    print('failed to save table, connection failed')
                    time.sleep(1)
                    self.cursor = self.connect()



import setuptools

setuptools.setup(
    name="data_integration_babel", 
    packages=setuptools.find_packages(exclude=["*.tests", "*.tests.*", "tests.*", "tests"]),
    version="0.2",
    license='MIT',
    author="weis",
    author_email="shaowei@funstory.ai",
    description="data_integration_babel",
    long_description="",
    long_description_content_type="text/markdown",
    url="",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
    ],
    keywords=['data_integration_babel'],
    python_requires='>=3.7',
    install_requires=[
        "pymysql==0.10.0",
        "kafka-python==2.0.1"
    ],
)